package com.jh.boot;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;

import com.jh.boot.email.LocalEmailService;
import com.jh.boot.security.utils.TemplateEmailAuthServiceListener;
import com.jh.boot.template.TemplateEngine;
import com.jh.boot.template.TemplateLoader;
import com.jh.boot.template.TemplateService;
import com.jh.boot.template.customizable.CustomizableLabelsService;
import com.jh.boot.template.customizable.CustomizableTemplateLoader;
import com.jh.boot.template.customizable.CustomizableTemplateService;
import com.jh.boot.template.handlebars.HandlebarsTemplateEngine;

public class JhBootApplication {

	@Bean
    @ConditionalOnProperty(value = "email.local.enabled", havingValue = "true", matchIfMissing = true)
	public LocalEmailService emailService() {
		return new LocalEmailService();
	}

	
    @Bean
    @ConditionalOnProperty(value = "security.auth.listener.email.enabled", havingValue = "true", matchIfMissing = true)
    public TemplateEmailAuthServiceListener authServiceListener() {
    	return new TemplateEmailAuthServiceListener();
    }
    
    @Bean
    public TemplateEngine templateEngine() {
    	return new HandlebarsTemplateEngine();
    }
    
    @Bean
    public TemplateService templateService() {
    	return new CustomizableTemplateService();
    }
    
    @Bean
    public CustomizableLabelsService labelsService() {
    	return new CustomizableLabelsService();
    }
    
    @Bean
    public TemplateLoader templateLoader() {
    	return new CustomizableTemplateLoader();
    }

}
