package com.jh.boot.security;

import java.io.IOException;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.context.annotation.Bean;
import org.springframework.security.authentication.AuthenticationProvider;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.AuthenticationEntryPoint;

import com.jh.boot.security.repository.AppUserRepository;
import com.jh.boot.security.service.AppUserAuthService;

/**
 * The Class ReactApplicationSecurityContext.
 */
public class JhSecurityConfig extends WebSecurityConfigurerAdapter {
    
    /**
     * @param http
     * @throws Exception
     * @see org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter#configure(org.springframework.security.config.annotation.web.builders.HttpSecurity)
     */
    @Override
    protected void configure(HttpSecurity http) throws Exception {
        HttpSecurity cfg = http
	        .authorizeRequests()
	        	.antMatchers("/auth/**").permitAll()
	        	.anyRequest().authenticated()
	        .and()
	        .logout()
	        	.clearAuthentication(true)
	            .permitAll()
            .and();
		
        cfg.httpBasic();
        
        http.csrf().disable();
        http.exceptionHandling().authenticationEntryPoint(authenticationEntryPoint());
        http.headers().frameOptions().disable();
        
        http.sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS);
    }

    
    /**
     * @param auth
     * @throws Exception
     * @see org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter#configure(org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder)
     */
    @Override
    protected void configure(AuthenticationManagerBuilder auth) throws Exception {
        auth.authenticationProvider(authenticationProvider());
    }    

    
    /**
     * Override this method to provide custom {@link AuthenticationEntryPoint}. 
     * 
     * It returns {@link RestAuthenticationEntryPoint} by default
     *
     * @return the authentication entry point
     */
    protected AuthenticationEntryPoint authenticationEntryPoint() {
    	return new AuthenticationEntryPoint() {
    	    @Override
    	    public final void commence(HttpServletRequest request, HttpServletResponse response, AuthenticationException authException) throws IOException {
    	        response.sendError(HttpServletResponse.SC_UNAUTHORIZED, "Unauthorized");
    	    }    		
    	};
    }

    @Bean
    public AuthenticationProvider authenticationProvider() {
    	return new AuthServiceAuthenticationProvider();
    }
   
    
    @Bean
    public AuthService authService() {
    	AppUserAuthService appUserAuthService = new AppUserAuthService();
    	appUserAuthService.setAppUserRepository(appUserRepository());
		return appUserAuthService;
    }
    
    @Bean
    public AppUserRepository appUserRepository() {
    	return new AppUserRepository();
    }
    
//    @Bean
//    public GrantedAuthorityDefaults grantedAuthorityDefaults() {
//        return new GrantedAuthorityDefaults(""); // Remove the ROLE_ prefix
//    }
    
}
