package com.jh.boot.security.utils;

import java.util.Collections;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.StringUtils;

import com.jh.boot.email.Email;
import com.jh.boot.email.EmailRecipient;
import com.jh.boot.email.EmailService;
import com.jh.boot.email.RecipientType;
import com.jh.boot.security.AuthServiceListener;
import com.jh.boot.security.model.AppUser;
import com.jh.boot.security.model.ResetPasswordToken;
import com.jh.boot.template.TemplateService;


public class TemplateEmailAuthServiceListener implements AuthServiceListener {

	@Autowired
	private EmailService emailService;
	
	@Autowired
	private TemplateService templateService;
	
	@Value("${template.email.layout:email.layout}")
	private String layoutTemplateCode;
	
	@Value("${template.email.signup.subject:email.signup.subject}")
	private String signupSubjectCode;

	@Value("${template.email.signup.text:template.email.signup.text}")
	private String signupTemplateCode;

	@Value("${template.email.signup.html:template.email.signup.html}")
	private String signupTemplateHtmlCode;

	@Value("${template.email.password.subject:email.password.subject}")
	private String passwordSubjectCode;

	@Value("${template.email.password.text:email.password.text}")
	private String passwordTemplateCode;

	@Value("${template.email.password.html:email.password.html}")
	private String passwordTemplateHtmlCode;
	
	
	@Override
	public void registerUser(AppUser user) {
		sendEmail(signupSubjectCode, signupTemplateCode, signupTemplateHtmlCode, user, Collections.singletonMap("user", user));
	}

	
	@Override
	public void generateResetToken(AppUser user, ResetPasswordToken token) {
		Map<String, Object> data = new HashMap<>();
		data.put("user", user);
		data.put("token", token.getToken());
		sendEmail(passwordSubjectCode, passwordTemplateCode, passwordTemplateHtmlCode, user, data);
	}
	
	
	@Override
	public void deleteUser(AppUser user) {
		//TODO
	}
	
	
	protected void sendEmail(String subject, String content, String html, AppUser user, Map<String, Object> data) {
		Locale locale = user.getLocale() == null ? Locale.getDefault() : new Locale(user.getLocale());

		subject = templateService.evaluate(subject, data, locale);
		content = templateService.evaluate(content, data, locale);
		html = templateService.evaluate(html, data, locale);
		
		//is layout defined?
		String layout = templateService.evaluate(layoutTemplateCode, Collections.singletonMap("content", html), locale);
		
		Email email = new Email();
		email.setRecipients(Collections.singleton(new EmailRecipient(user.getEmail(), RecipientType.TO)));
		email.setSubject(subject);
		email.setText(content);
		email.setHtml(StringUtils.hasText(layout) ? layout : html);

		emailService.sendEmail(email);
	}

}
